@@ -37,6 +37,8 @@ impl TreeSitterParser {
                 .flag_if_supported("-Wno-implicit-fallthrough")
                 .flag_if_supported("-Wno-unused-parameter")
                 .flag_if_supported("-Wno-ignored-qualifiers")
+                // Ignore warning from tree-sitter-html.
+                .flag_if_supported("-Wno-sign-compare")
                 // Workaround for: https://github.com/ganezdragon/tree-sitter-perl/issues/16
                 // should be removed after fixed.
                 .flag_if_supported("-Wno-return-type");
@@ -137,6 +139,11 @@ fn main() {
             src_dir: "vendor/tree-sitter-hcl-src",
             extra_files: vec!["scanner.cc"],
         },
+        TreeSitterParser {
+            name: "tree-sitter-html",
+            src_dir: "vendor/tree-sitter-html-src",
+            extra_files: vec!["scanner.cc"],
+        },
         TreeSitterParser {
             name: "tree-sitter-janet-simple",
             src_dir: "vendor/tree-sitter-janet-simple-src",
@@ -157,6 +164,11 @@ fn main() {
             src_dir: "vendor/tree-sitter-json-src",
             extra_files: vec![],
         },
+        TreeSitterParser {
+            name: "tree-sitter-julia",
+            src_dir: "vendor/tree-sitter-julia-src",
+            extra_files: vec!["scanner.c"],
+        },
         TreeSitterParser {
             name: "tree-sitter-kotlin",
             src_dir: "vendor/tree-sitter-kotlin-src",
