@@ -64,10 +64,9 @@ fn prefer_outer_delimiter(language: guess_language::Language) -> bool {
         // languages have syntax like `foo(bar)` or `foo[bar]` where
         // the inner delimiter is more relevant.
         Bash | C | CPlusPlus | CSharp | Css | Dart | Elixir | Elm | Elvish | Gleam | Go
-        | Haskell | Java | JavaScript | Jsx | Kotlin | Lua | Nix | OCaml | OCamlInterface
-        | Perl | Php | Python | Ruby | Rust | Scala | Swift | Tsx | TypeScript | Yaml | Zig => {
-            false
-        }
+        | Haskell | Html | Java | JavaScript | Jsx | Julia | Kotlin | Lua | Nix | OCaml
+        | OCamlInterface | Perl | Php | Python | Ruby | Rust | Scala | Swift | Tsx | TypeScript
+        | Yaml | Zig => false,
     }
 }
 
@@ -538,7 +537,7 @@ fn slide_to_next_node<'a>(
 
 /// Return the distance between two syntax nodes, as a tuple of number
 /// of lines and number of columns.
-fn distance_between(prev: &Syntax, next: &Syntax) -> (usize, usize) {
+fn distance_between(prev: &Syntax, next: &Syntax) -> (u32, u32) {
     let prev_pos = prev.last_line_span();
     let next_pos = next.first_line_span();
 
