@@ -4,7 +4,7 @@ use crate::{
     display::context::{calculate_after_context, calculate_before_context, opposite_positions},
     display::hunks::Hunk,
     display::style::{self, apply_colors},
-    lines::{format_line_num, MaxLine},
+    lines::{format_line_num, split_on_newlines, MaxLine},
     options::DisplayOptions,
     parse::syntax::MatchedPos,
 };
@@ -21,7 +21,7 @@ pub fn print(
     rhs_display_path: &str,
     lang_name: &str,
 ) {
-    let (lhs_colored, rhs_colored) = if display_options.use_color {
+    let (lhs_colored_lines, rhs_colored_lines) = if display_options.use_color {
         (
             apply_colors(
                 lhs_src,
@@ -39,12 +39,18 @@ pub fn print(
             ),
         )
     } else {
-        (lhs_src.to_string(), rhs_src.to_string())
+        (
+            split_on_newlines(lhs_src)
+                .iter()
+                .map(|s| (*s).to_owned())
+                .collect(),
+            split_on_newlines(rhs_src)
+                .iter()
+                .map(|s| (*s).to_owned())
+                .collect(),
+        )
     };
 
-    let lhs_lines: Vec<_> = lhs_colored.lines().collect();
-    let rhs_lines: Vec<_> = rhs_colored.lines().collect();
-
     let opposite_to_lhs = opposite_positions(lhs_positions);
     let opposite_to_rhs = opposite_positions(rhs_positions);
 
@@ -76,7 +82,11 @@ pub fn print(
 
         for (lhs_line, _) in before_lines {
             if let Some(lhs_line) = lhs_line {
-                println!("{}   {}", format_line_num(lhs_line), lhs_lines[lhs_line.0]);
+                println!(
+                    "{}   {}",
+                    format_line_num(lhs_line),
+                    lhs_colored_lines[lhs_line.as_usize()]
+                );
             }
         }
 
@@ -85,7 +95,7 @@ pub fn print(
                 println!(
                     "{}   {}",
                     format_line_num(*lhs_line).red().bold(),
-                    lhs_lines[lhs_line.0]
+                    lhs_colored_lines[lhs_line.as_usize()]
                 );
             }
         }
@@ -94,14 +104,18 @@ pub fn print(
                 println!(
                     "   {}{}",
                     format_line_num(*rhs_line).green().bold(),
-                    rhs_lines[rhs_line.0]
+                    rhs_colored_lines[rhs_line.as_usize()]
                 );
             }
         }
 
         for (_, rhs_line) in &after_lines {
             if let Some(rhs_line) = rhs_line {
-                println!("   {}{}", format_line_num(*rhs_line), rhs_lines[rhs_line.0]);
+                println!(
+                    "   {}{}",
+                    format_line_num(*rhs_line),
+                    rhs_colored_lines[rhs_line.as_usize()]
+                );
             }
         }
         println!();
