@@ -59,10 +59,12 @@ extern "C" {
     fn tree_sitter_go() -> ts::Language;
     fn tree_sitter_haskell() -> ts::Language;
     fn tree_sitter_hcl() -> ts::Language;
+    fn tree_sitter_html() -> ts::Language;
     fn tree_sitter_janet_simple() -> ts::Language;
     fn tree_sitter_java() -> ts::Language;
     fn tree_sitter_javascript() -> ts::Language;
     fn tree_sitter_json() -> ts::Language;
+    fn tree_sitter_julia() -> ts::Language;
     fn tree_sitter_kotlin() -> ts::Language;
     fn tree_sitter_lua() -> ts::Language;
     fn tree_sitter_nix() -> ts::Language;
@@ -345,6 +347,30 @@ pub fn from_language(language: guess::Language) -> TreeSitterConfig {
                 .unwrap(),
             }
         }
+        Html => {
+            let language = unsafe { tree_sitter_html() };
+            TreeSitterConfig {
+                name: "HTML",
+                language,
+                atom_nodes: vec![
+                    "quoted_attribute_value",
+                    "comment",
+                    "raw_text",
+                    "tag_name",
+                    "text",
+                ]
+                .into_iter()
+                .collect(),
+                delimiter_tokens: vec![("<", ">"), ("<!", ">"), ("<!--", "-->")]
+                    .into_iter()
+                    .collect(),
+                highlight_query: ts::Query::new(
+                    language,
+                    include_str!("../../vendor/highlights/html.scm"),
+                )
+                .unwrap(),
+            }
+        }
         Janet => {
             let language = unsafe { tree_sitter_janet_simple() };
             TreeSitterConfig {
@@ -420,6 +446,27 @@ pub fn from_language(language: guess::Language) -> TreeSitterConfig {
                 .unwrap(),
             }
         }
+        Julia => {
+            let language = unsafe { tree_sitter_julia() };
+            TreeSitterConfig {
+                name: "Julia",
+                language,
+                atom_nodes: vec![
+                    "string_literal",
+                    "prefixed_string_literal",
+                    "command_literal",
+                    "character_literal",
+                ]
+                .into_iter()
+                .collect(),
+                delimiter_tokens: vec![("{", "}"), ("[", "]"), ("(", ")")],
+                highlight_query: ts::Query::new(
+                    language,
+                    include_str!("../../vendor/highlights/julia.scm"),
+                )
+                .unwrap(),
+            }
+        }
         Kotlin => {
             let language = unsafe { tree_sitter_kotlin() };
             TreeSitterConfig {
@@ -753,6 +800,7 @@ fn tree_highlights(
             || name.starts_with("constant.")
             || name == "operator"
             || name == "repeat"
+            || name == "conditional"
             || name == "boolean"
         {
             keyword_ish_capture_ids.push(idx as u32);
